@@ -1,5 +1,5 @@
 #line 1
-# vim: set ts=4 sts=4 sw=4 expandtab smarttab:
+# vim: set ts=2 sts=2 sw=2 expandtab smarttab:
 #
 # This file is part of Pod-Markdown
 #
@@ -14,24 +14,106 @@ use warnings;
 
 package Pod::Markdown;
 {
-  $Pod::Markdown::VERSION = '1.321';
+  $Pod::Markdown::VERSION = '1.500';
 }
-# git description: v1.320-6-g35fc58e
+# git description: v1.401-20-g3999377
 
 BEGIN {
   $Pod::Markdown::AUTHORITY = 'cpan:RWSTAUNER';
 }
 # ABSTRACT: Convert POD to Markdown
+
+use Pod::Parser 1.51 ();
 use parent qw(Pod::Parser);
 use Pod::ParseLink (); # core
 
+our %URL_PREFIXES = (
+  sco      => 'http://search.cpan.org/perldoc?',
+  metacpan => 'https://metacpan.org/pod/',
+  man      => 'http://man.he.net/man',
+);
+$URL_PREFIXES{perldoc} = $URL_PREFIXES{metacpan};
+
+
+# new() is provided by Pod::Parser (which calls initialize()).
+
 sub initialize {
     my $self = shift;
     $self->SUPER::initialize(@_);
+
+    for my $type ( qw( perldoc man ) ){
+        my $attr  = $type . '_url_prefix';
+        # Use provided argument or default alias.
+        my $url = $self->{ $attr } || $type;
+        # Expand alias if defined (otherwise use url as is).
+        $self->{ $attr } = $URL_PREFIXES{ $url } || $url;
+    }
+
+    $self->_prepare_fragment_formats;
+
     $self->_private;
     $self;
 }
 
+
+my @attr = qw(
+  man_url_prefix
+  perldoc_url_prefix
+  perldoc_fragment_format
+  markdown_fragment_format
+);
+
+{
+  no strict 'refs'; ## no critic
+  foreach my $attr ( @attr ){
+    *$attr = sub { return $_[0]->{ $attr } };
+  }
+}
+
+sub _prepare_fragment_formats {
+  my ($self) = @_;
+
+  foreach my $attr ( @attr ){
+    next unless $attr =~ /^(\w+)_fragment_format/;
+    my $type = $1;
+    my $format = $self->{ $attr };
+
+    # If one was provided.
+    if( $format ){
+      # If the attribute is a coderef just use it.
+      next if ref($format) eq 'CODE';
+    }
+    # Else determine a default.
+    else {
+      if( $type eq 'perldoc' ){
+        # Choose a default that matches the destination url.
+        my $target = $self->{perldoc_url_prefix};
+        foreach my $alias ( qw( metacpan sco ) ){
+          if( $target eq $URL_PREFIXES{ $alias } ){
+            $format = $alias;
+          }
+        }
+        # This seems like a reasonable fallback.
+        $format ||= 'pod_simple_xhtml';
+      }
+      else {
+        $format = $type;
+      }
+    }
+
+    # The short name should become a method name with the prefix prepended.
+    my $prefix = 'format_fragment_';
+    $format =~ s/^$prefix//;
+    die "Unknown fragment format '$format'"
+      unless $self->can($prefix . $format);
+
+    # Save it.
+    $self->{ $attr } = $format;
+  }
+
+  return;
+}
+
 sub _private {
     my $self = shift;
     $self->{_MyParser} ||= {
@@ -45,6 +127,7 @@ sub _private {
     };
 }
 
+
 sub as_markdown {
     my ($parser, %args) = @_;
     my $data  = $parser->_private;
@@ -102,6 +185,7 @@ sub _clean_text {
     return wantarray ? @trimmed : join("\n", @trimmed);
 }
 
+# Backslash escape markdown characters to avoid having them interpreted.
 sub _escape {
     local $_ = $_[1];
 
@@ -117,6 +201,15 @@ sub _escape {
     return $_;
 }
 
+# Formats a header according to the given level.
+sub format_header {
+    my ($self, $level, $paragraph) = @_;
+    # TODO: put a name="" if $self->{embed_anchor_tags}; ?
+    # https://rt.cpan.org/Ticket/Display.html?id=57776
+    sprintf '%s %s', '#' x $level, $paragraph;
+}
+
+# Handles POD command paragraphs, denoted by a line beginning with C<=>.
 sub command {
     my ($parser, $command, $paragraph, $line_num) = @_;
     my $data = $parser->_private;
@@ -128,8 +221,7 @@ sub command {
     if ($command =~ m{head(\d)}xms) {
         my $level = $1;
 
-        $paragraph = $parser->_escape($paragraph);
-        $paragraph = $parser->interpolate($paragraph, $line_num);
+        $paragraph = $parser->_escape_and_interpolate($paragraph, $line_num);
 
         # the headers never are indented
         $parser->_save($parser->format_header($level, $paragraph));
@@ -166,9 +258,9 @@ sub command {
         $data->{ListType} = '-'; # Default
         if($paragraph =~ m{^[ \t]* \* [ \t]*}xms) {
             $paragraph =~ s{^[ \t]* \* [ \t]*}{}xms;
-        } elsif($paragraph =~ m{^[ \t]* (\d+\.) [ \t]*}xms) {
-            $data->{ListType} = $1; # For numbered list only
-            $paragraph =~ s{^[ \t]* \d+\. [ \t]*}{}xms;
+        } elsif($paragraph =~ m{^[ \t]* (\d+)\.? [ \t]*}xms) {
+            $data->{ListType} = $1.'.'; # For numbered list only
+            $paragraph =~ s{^[ \t]* \d+\.? [ \t]*}{}xms;
         }
 
         if ($data->{searching} eq 'listpara') {
@@ -187,6 +279,7 @@ sub command {
     return;
 }
 
+# Handles verbatim text.
 sub verbatim {
     my ($parser, $paragraph) = @_;
 
@@ -211,20 +304,23 @@ sub verbatim {
         $paragraph = join "\n", map { /^\t/ ? $_ : $indent . $_ } @lines;
     }
 
+    # FIXME: Checking _PREVIOUS is breaking Pod::Parser encapsulation
+    # but helps solve the extraneous extra blank line b/t verbatim blocks.
+    # We could probably keep track ourselves if need be.
+    # NOTE: This requires Pod::Parser 1.50.
+    # This is another reason to switch to Pod::Simple.
+    my $previous_was_verbatim =
+        $parser->{_PREVIOUS} && $parser->{_PREVIOUS} eq 'verbatim';
+
+    if($previous_was_verbatim && $parser->_private->{Text}->[-1] =~ /[ \t]+$/){
+        $paragraph = $parser->_unsave . "\n" . $paragraph;
+    }
+
     $parser->_save($paragraph);
 }
 
-sub _escape_non_code {
-    my ($parser, $text, $ptree) = @_;
-    $text = $parser->_escape($text)
-        unless $ptree->isa('Pod::InteriorSequence') && $ptree->cmd_name eq 'C';
-    return $text;
-}
-
-sub textblock {
+sub _escape_and_interpolate {
     my ($parser, $paragraph, $line_num) = @_;
-    my $data = $parser->_private;
-    my $prelisthead;
 
     # escape markdown characters in text sequences except for inline code
     $paragraph = join '', $parser->parse_text(
@@ -235,6 +331,26 @@ sub textblock {
     # interpolate the paragraph for embedded sequences
     $paragraph = $parser->interpolate($paragraph, $line_num);
 
+    return $paragraph;
+}
+
+sub _escape_non_code {
+    my ($parser, $text, $ptree) = @_;
+
+    if ($ptree->isa('Pod::InteriorSequence') && $ptree->cmd_name =~ /\A[CFL]\z/) {
+        return $text;
+    }
+    return $parser->_escape($text);
+}
+
+# Handles normal blocks of POD.
+sub textblock {
+    my ($parser, $paragraph, $line_num) = @_;
+    my $data = $parser->_private;
+    my $prelisthead;
+
+    $paragraph = $parser->_escape_and_interpolate($paragraph, $line_num);
+
     # clean the empty lines
     $paragraph = $parser->_clean_text($paragraph);
 
@@ -260,6 +376,10 @@ sub textblock {
     $parser->_save($paragraph, $prelisthead);
 }
 
+# An interior sequence is an embedded command
+# within a block of text which appears as a command name - usually a single
+# uppercase character - followed immediately by a string of text which is
+# enclosed in angle brackets.
 sub interior_sequence {
     my ($self, $seq_command, $seq_argument, $pod_seq) = @_;
 
@@ -271,8 +391,8 @@ sub interior_sequence {
     my %interiors = (
         'I' => sub { return '_'  . $_[$i] . '_'  },      # italic
         'B' => sub { return '__' . $_[$i] . '__' },      # bold
-        'C' => sub { return '`'  . $_[$i] . '`'  },      # monospace
-        'F' => sub { return '`'  . $_[$i] . '`'  },      # system path
+        'C' => \&_wrap_code_span,                        # monospace
+        'F' => \&_wrap_code_span,                        # system path
         # non-breaking space
         'S' => sub {
             (my $s = $_[$i]) =~ s/ /&nbsp;/g;
@@ -297,6 +417,7 @@ sub interior_sequence {
             return "&$charname;";
         },
         'L' => \&_resolv_link,
+        # TODO: create `a name=` if configured?
         'X' => sub { '' },
         'Z' => sub { '' },
     );
@@ -319,26 +440,12 @@ sub _resolv_link {
       Pod::ParseLink::parselink($arg);
     my $url = '';
 
-    # TODO: make url prefixes configurable
-
     if ($type eq 'url') {
         $url = $name;
     } elsif ($type eq 'man') {
-        # stolen from Pod::Simple::(X)HTML
-        my ($page, $part) = $name =~ /\A([^(]+)(?:[(](\S*)[)])?/;
-        $url = 'http://man.he.net/man' . ($part || 1) . '/' . ($page || $name);
+        $url = $self->format_man_url($name);
     } else {
-        if ($name) {
-            $url = 'http://search.cpan.org/perldoc?' . $name;
-        }
-        if ($section){
-            # TODO: sites/pod formatters differ on how to transform the section
-            # TODO: we could do it according to specified url prefix or pod formatter
-            # TODO: or allow a coderef?
-            # TODO: (Pod::Simple::XHTML:idify() for metacpan)
-            # TODO: (Pod::Simple::HTML section_escape/unicode_escape_url/section_url_escape for s.c.o.)
-            $url .= '#' . $section;
-        }
+        $url = $self->format_perldoc_url($name, $section);
     }
 
     # if we don't know how to handle the url just print the pod back out
@@ -346,16 +453,164 @@ sub _resolv_link {
         return sprintf '%s<%s>', $cmd, $arg;
     }
 
+    # TODO: put unescaped section into link title? [a](b "c")
     return sprintf '[%s](%s)', ($text || $inferred), $url;
 }
 
-sub format_header {
-    my ($level, $paragraph) = @_[1,2];
-    sprintf '%s %s', '#' x $level, $paragraph;
+# A code span can be delimited by multiple backticks (and a space)
+# similar to pod codes (C<< code >>), so ensure we use a big enough
+# delimiter to not have it broken by embedded backticks.
+sub _wrap_code_span {
+  my ($self, undef, $arg) = @_;
+  my $longest = 0;
+  while( $arg =~ /([`]+)/g ){
+    my $len = length($1);
+    $longest = $len if $longest < $len;
+  }
+  my $delim = '`' x ($longest + 1);
+  my $pad = $longest > 0 ? ' ' : '';
+  return $delim . $pad . $arg . $pad . $delim;
 }
 
+
+sub format_man_url {
+    my ($self, $to) = @_;
+    my ($page, $part) = ($to =~ /^ ([^(]+) (?: \( (\S+) \) )? /x);
+    return $self->man_url_prefix . ($part || 1) . '/' . ($page || $to);
+}
+
+
+sub format_perldoc_url {
+  my ($self, $name, $section) = @_;
+
+  my $url_prefix = $self->perldoc_url_prefix;
+  my $url = '';
+
+  # If the link is to another module (external link).
+  if ($name) {
+    $url = $url_prefix . $name;
+  }
+
+  # See https://rt.cpan.org/Ticket/Display.html?id=57776
+  # for a discussion on the need to mangle the section.
+  if ($section){
+
+    my $method = $url
+      # If we already have a prefix on the url it's external.
+      ? $self->perldoc_fragment_format
+      # Else an internal link points to this markdown doc.
+      : $self->markdown_fragment_format;
+
+    $method = 'format_fragment_' . $method
+      unless ref($method);
+
+    {
+      # Set topic to enable code refs to be simple.
+      local $_ = $section;
+      $section = $self->$method($section);
+    }
+
+    $url .= '#' . $section;
+  }
+
+  return $url;
+}
+
+
+# TODO: simple, pandoc, etc?
+
+sub format_fragment_markdown {
+  my ($self, $section) = @_;
+
+  # If this is an internal link (to another section in this doc)
+  # we can't be sure what the heading id's will look like
+  # (it depends on what is rendering the markdown to html)
+  # but we can try to follow popular conventions.
+
+  # http://johnmacfarlane.net/pandoc/demo/example9/pandocs-markdown.html#header-identifiers-in-html-latex-and-context
+  #$section =~ s/(?![-_.])[[:punct:]]//g;
+  #$section =~ s/\s+/-/g;
+  $section =~ s/\W+/-/g;
+  $section =~ s/-+$//;
+  $section =~ s/^-+//;
+  $section = lc $section;
+  #$section =~ s/^[^a-z]+//;
+  $section ||= 'section';
+
+  return $section;
+}
+
+
+{
+  # From Pod::Simple::XHTML 3.28.
+  # The strings gets passed through encode_entities() before idify().
+  # If we don't do it here the substitutions below won't operate consistently.
+
+  # encode_entities {
+    my %entities = (
+      q{>} => 'gt',
+      q{<} => 'lt',
+      q{'} => '#39',
+      q{"} => 'quot',
+      q{&} => 'amp',
+    );
+
+    my
+      $ents = join '', keys %entities;
+  # }
+
+  sub format_fragment_pod_simple_xhtml {
+    my ($self, $t) = @_;
+
+    # encode_entities {
+      $t =~ s/([$ents])/'&' . ($entities{$1} || sprintf '#x%X', ord $1) . ';'/ge;
+    # }
+
+    # idify {
+      for ($t) {
+          s/<[^>]+>//g;            # Strip HTML.
+          s/&[^;]+;//g;            # Strip entities.
+          s/^\s+//; s/\s+$//;      # Strip white space.
+          s/^([^a-zA-Z]+)$/pod$1/; # Prepend "pod" if no valid chars.
+          s/^[^a-zA-Z]+//;         # First char must be a letter.
+          s/[^-a-zA-Z0-9_:.]+/-/g; # All other chars must be valid.
+          s/[-:.]+$//;             # Strip trailing punctuation.
+      }
+    # }
+
+    return $t;
+  }
+}
+
+
+sub format_fragment_pod_simple_html {
+  my ($self, $section) = @_;
+
+  # From Pod::Simple::HTML 3.28.
+
+  # section_name_tidy {
+    $section =~ s/^\s+//;
+    $section =~ s/\s+$//;
+    $section =~ tr/ /_/;
+    $section =~ tr/\x00-\x1F\x80-\x9F//d if 'A' eq chr(65); # drop crazy characters
+
+    #$section = $self->unicode_escape_url($section);
+      # unicode_escape_url {
+      $section =~ s/([^\x00-\xFF])/'('.ord($1).')'/eg;
+        #  Turn char 1234 into "(1234)"
+      # }
+
+    $section = '_' unless length $section;
+    return $section;
+  # }
+}
+
+
+sub format_fragment_metacpan { shift->format_fragment_pod_simple_xhtml(@_); }
+sub format_fragment_sco      { shift->format_fragment_pod_simple_html(@_);  }
+
 1;
 
 __END__
 
-#line 546
+#line 936
